package be.cylab.java.wowa.training;

import org.deeplearning4j.nn.api.OptimizationAlgorithm;
import org.nd4j.linalg.activations.Activation;

/**
 * Parameters of the NeuralNetwork class.
 */
public class HyperParameters {
    private int neurons_number;
    private double learning_rate;
    private OptimizationAlgorithm algorithm;
    private Activation activation_function;
    private double percent_test_train;

    /**
     * Default constructor.
     * @param neurons_number
     * @param learning_rate
     * @param algorithm
     * @param activation_function
     * @param percent_test_train
     */
    public HyperParameters(
            final int neurons_number,
            final double learning_rate,
            final OptimizationAlgorithm algorithm,
            final Activation activation_function,
            final double percent_test_train) {
        setNeuronsNumber(neurons_number);
        setLearningRate(learning_rate);
        setAlgorithm(algorithm);
        setActivationFunction(activation_function);
        setPercentTestTrain(percent_test_train);
    }

    /**
     * Constructor without percent_train_test value.
     * @param neurons_number
     * @param learning_rate
     * @param algorithm
     * @param activation_function
     */
    public HyperParameters(
            final int neurons_number,
            final double learning_rate,
            final OptimizationAlgorithm algorithm,
            final Activation activation_function) {
        this(neurons_number, learning_rate, algorithm,
                activation_function, 99);
    }

    /**
     * Getter for neuron_number.
     * @return
     */
    public final int getNeuronsNumber() {
        return neurons_number;
    }

    /**
     * Getter for learning rate.
     * @return
     */
    public final double getLearningRate() {
        return learning_rate;
    }

    /**
     * Getter for backpropagation algorithm.
     * @return
     */
    public final OptimizationAlgorithm getAlgorithm() {
        return algorithm;
    }

    /**
     * Getter for activation function.
     * @return
     */
    public final Activation getActivationFunction() {
        return activation_function;
    }

    /**
     * Getter for percent_test_train.
     * @return
     */
    public final double getPercentTestTrain() {
        return percent_test_train;
    }

    /**
     * @param neurons_number
     */
    public final void setNeuronsNumber(final int neurons_number) {
        if (neurons_number < 5 || neurons_number > 80) {
            throw new IllegalArgumentException(
                    "Neuron number must be between 5 and 80");
        }
        this.neurons_number = neurons_number;
    }

    /**
     * @param learning_rate
     */
    public final void setLearningRate(final double learning_rate) {
        if (learning_rate <= 0.0 || learning_rate >= 1.0) {
            throw new IllegalArgumentException(
                    "Learning rate must be between 0 and 1");
        }
        this.learning_rate = learning_rate;
    }

    /**
     * @param algorithm
     */
    public final void setAlgorithm(final OptimizationAlgorithm algorithm) {
        this.algorithm = algorithm;
    }

    /**
     * @param activation_function
     */
    public final void setActivationFunction(
            final Activation activation_function) {
        this.activation_function = activation_function;
    }

    /**
     * @param percent_test_train
     */
    public final void setPercentTestTrain(final double percent_test_train) {
        if (percent_test_train <= 0 || percent_test_train >= 100) {
            throw new IllegalArgumentException(
                    "Percentage of train must be between 0 and 100");
        }
        this.percent_test_train = percent_test_train;
    }
}
