package be.cylab.java.wowa.training;

import org.deeplearning4j.nn.api.OptimizationAlgorithm;
import org.deeplearning4j.nn.multilayer.MultiLayerNetwork;
import org.nd4j.linalg.activations.Activation;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;

/**
 * Class for learn in neuronal network.
 */
public final class MainDL4J {
    /**
     * Default constructor.
     */
    private MainDL4J() {

    }

    /**
     * Main class for neural-network learning.
     *
     * @param args
     */
    public static void main(final String[] args) {
        int begin_neuron_number = Integer.parseInt(args[0]);
        int neuron_number_end = Integer.parseInt(args[1]);
        String data_file = args[5];
        String expected_file = args[6];
        double learning_rate = Double.parseDouble(args[2]);
        OptimizationAlgorithm optimization_algorithm;
        Activation activation_function;
        int fold_number = 10;
        int increase_ratio = 10;

        if (args[3].matches("CONJUGATE_GRADIENT")) {
            optimization_algorithm = OptimizationAlgorithm.CONJUGATE_GRADIENT;
        } else if (args[3].matches("LBFGS")) {
            optimization_algorithm = OptimizationAlgorithm.LBFGS;
        } else if (args[3].matches("LINE_GRADIENT_DESCENT")) {
            optimization_algorithm
                    = OptimizationAlgorithm.LINE_GRADIENT_DESCENT;
        } else if (args[3].matches("STOCHASTIC_GRADIENT_DESCENT")) {
            optimization_algorithm
                    = OptimizationAlgorithm.STOCHASTIC_GRADIENT_DESCENT;
        } else {
            throw new IllegalArgumentException(
                    "Not correct optimization algorithm");
        }

        if (args[4].matches("RELU")) {
            activation_function = Activation.RELU;
        } else if (args[4].matches("SIGMOID")) {
            activation_function = Activation.SIGMOID;
        } else if (args[4].matches("TANH")) {
            activation_function = Activation.TANH;
        } else {
            throw new IllegalArgumentException(
                    "Not correct activation function");
        }

        for (int neuron_number = begin_neuron_number;
             neuron_number < neuron_number_end; neuron_number++) {
            double average_auc = 0;
            System.out.println("Neuron number : " + neuron_number);
            HyperParameters parameters
                    = new HyperParameters(neuron_number, learning_rate,
                    optimization_algorithm, activation_function);
            NeuralNetwork nn = new NeuralNetwork(parameters);

            HashMap<MultiLayerNetwork, Double> map = nn.runKFold(
                    data_file,
                    expected_file,
                    fold_number,
                    increase_ratio);
            for (Double d : map.values()) {
                average_auc = average_auc + d;
            }
            try (OutputStreamWriter writer = new OutputStreamWriter(
                    new FileOutputStream("Synthesis_average_AUC.txt", true),
                    StandardCharsets.UTF_8)) {
                writer.write("Neuron number : " + neuron_number
                        + " Learning rate : " + learning_rate
                        + " Algorithm : " + args[3]
                        + " Activation function : " + args[4]
                        + " Average AUC = "
                        + (average_auc / fold_number) + "\n");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }


    }
}
