package be.cylab.java.wowa.training;

import org.deeplearning4j.nn.api.OptimizationAlgorithm;
import org.deeplearning4j.nn.multilayer.MultiLayerNetwork;
import org.nd4j.linalg.activations.Activation;

import java.util.HashMap;
import java.util.List;

/**
 * Main class for test and compare efficiency of nn and wt.
 */
public final class MainTest {

    private MainTest() {

    }

    /**
     * @param args
     */
    public static void main(final String[] args) {
        int population_size = 100;
        int crossover_rate = 60;
        int muatation_rate = 15;
        int max_generation_number = 120;
        int selection_method
                = TrainerParameters.SELECTION_METHOD_RWS;
        int generation_population_method
                = TrainerParameters.POPULATION_INITIALIZATION_RANDOM;
        String data_file = "ressources/webshell_data.json";
        String expected_file = "ressources/webshell_expected.json";
        List<List<Double>> data
                = Utils.convertJsonToDataForTrainer(data_file);
        List<Double> expected
                = Utils.convertJsonToExpectedForTrainer(expected_file);

        TrainerParameters wt_parameters = new TrainerParameters(
                null,
                population_size,
                crossover_rate,
                muatation_rate,
                max_generation_number,
                selection_method,
                generation_population_method);
        Trainer trainer = new Trainer(wt_parameters,
                new SolutionDistance(5));

        int neurons_number = 20;
        double learning_rate = 0.01;
        OptimizationAlgorithm algo = OptimizationAlgorithm.CONJUGATE_GRADIENT;
        Activation activation_function = Activation.TANH;
        HyperParameters nn_parameters = new HyperParameters(
                neurons_number,
                learning_rate,
                algo,
                activation_function
        );
        NeuralNetwork nn = new NeuralNetwork(nn_parameters);

        TrainingDataset dataset = new TrainingDataset(data, expected);
        List<TrainingDataset> folds = dataset.prepareFolds(10);

        long start_time = System.currentTimeMillis();
        System.out.println("Wowa training");
        HashMap<AbstractSolution, Double> map_wt = trainer.runKFold(folds, 10);
        long end_time = System.currentTimeMillis();
        System.out.println("Execution time : " + (end_time - start_time) / 1000
                + " seconds");

        start_time = System.currentTimeMillis();
        System.out.println("Neural Network learning");
        HashMap<MultiLayerNetwork, Double> map_nn = nn.runKFold(folds, 10);
        end_time = System.currentTimeMillis();
        System.out.println("Execution time : " + (end_time - start_time) / 1000
                + " seconds");

        double nn_score = 0.0;
        double wt_score = 0.0;
        for (Double d : map_nn.values()) {
            nn_score = nn_score + d;
        }

        System.out.println("Average AUC for Neural Network learning : "
                + nn_score / 10);

        for (Double d : map_wt.values()) {
            wt_score = wt_score + d;
        }

        System.out.println("Average AUC for WOWA learning : " + wt_score / 10);
    }
}
